<?php

namespace Tablesome\Components\Table;

use \Tablesome\Includes\Settings\Tablesome_Getter;

if (!class_exists('\Tablesome\Components\Table\Model')) {
    class Model
    {

        public function __construct()
        {
            $this->tablesome_db = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();
        }

        public function get_viewProps($args = [])
        {
            $this->collection = $this->get_collectionProps($args);
            $this->collection = array_merge($this->collection, $args);

            return [
                'collection' => $this->collection,
                'items' => $this->get_itemProps($this->collection),
            ];
        }

        public function get_collectionProps($args = [])
        {
            $this->table_id = isset($args['post_id']) ? $args['post_id'] : $args['table_id'];
            $this->table_meta = get_tablesome_data($this->table_id);

            $this->columns = isset($this->table_meta['columns']) && !empty($this->table_meta['columns']) ? $this->table_meta['columns'] : [];

            $last_column_id = isset($this->table_meta['meta']['last_column_id']) && !empty($this->table_meta['meta']['last_column_id']) ? $this->table_meta['meta']['last_column_id'] : 1;
            $table_mode = isset($args['table_mode']) ? $args['table_mode'] : 'read-only';
            $custom_style = Tablesome_Getter::get('style_disable') ? true : false;

            $collection = [
                'table_title' => $this->get_table_title($this->table_id),
                'rowLimit' => TABLESOME_MAX_RECORDS_TO_READ,
                'columnLimit' => TABLESOME_MAX_COLUMNS_TO_READ,
                'defaultColumnName' => __("Column name", 'tablesome'),
                'rowDeleteMessage' => __("Are you sure want's to Delete the selected Row?", 'tablesome'),
                'columnDeleteMessage' => __("Are you sure want's to Delete the selected Column?", 'tablesome'),
                'rowLimitMessage' => __("Couldn't remove the row. The table must have one row at least!", 'tablesome'),
                'columnLimitMessage' => __("Couldn't remove the Column. The table must have one Column at least!", 'tablesome'),
                'last_column_id' => $last_column_id,
                'show_table_footer' => isset($args['show_table_footer']) ? $args['show_table_footer'] : false,
                'last_rank_order' => $this->tablesome_db->get_max_rank_order_value($this->table_id),
                'mode' => $table_mode,
                'customstyle' => $custom_style,
                'display' => $this->get_display_settings($this->table_meta, $args),
            ];

            return $collection;
        }

        public function get_display_settings($meta = [], $args = [])
        {
            $options = isset($meta['options']) && !empty($meta['options']) ? $meta['options'] : [];
            $tableLevelDisplay = isset($options['display']) && !empty($options['display']) ? $options['display'] : [];

            $general = [
                'displayMode' => Tablesome_Getter::get('table_display_mode'),
                'mobileLayoutMode' => Tablesome_Getter::get('mobile_layout_mode'),
                'numOfRecordsPerPage' => strval(Tablesome_Getter::get('num_of_records_per_page')),
                'hideTableHeader' => $this->getBool(Tablesome_Getter::get('hide_table_header')),
                'stickyFirstColumn' => $this->getBool(Tablesome_Getter::get('sticky_first_column')),
                'serialNumberColumn' => $this->getBool(Tablesome_Getter::get('show_serial_number_column')),

                'enableColumnMinWidth' => $this->getBool(Tablesome_Getter::get('enable_min_column_width')),
                'enableColumnMaxWidth' => $this->getBool(Tablesome_Getter::get('enable_max_column_width')),

                'columnMinWidth' => strval(Tablesome_Getter::get('min_column_width')),
                'columnMaxWidth' => strval(Tablesome_Getter::get('max_column_width')),
                // 'sortingOrder' => 'asc',
                // 'sortingBy' => 'rank_order',

            ];

            $generalShowHideComponent = [
                "showHideComponentDevice" => "desktop",
                // desktop
                'desktop-search' => $this->getBool(Tablesome_Getter::get('search')),
                'desktop-sort' => $this->getBool(Tablesome_Getter::get('sorting')),
                'desktop-filter' => $this->getBool(Tablesome_Getter::get('filters')),

                // mobile
                'mobile-search' => $this->getBool(Tablesome_Getter::get('search')),
                'mobile-sort' => $this->getBool(Tablesome_Getter::get('sorting')),
                'mobile-filter' => $this->getBool(Tablesome_Getter::get('filters')),

            ];

            $general = array_merge($general, $generalShowHideComponent);
            $arguments = [];
            // error_log(' args : ' . print_r($args, true));

            // shortcode arguments or widget arguments
            if (isset($args['table_display_mode']) && !empty($args['table_display_mode'])) {
                $arguments['displayMode'] = $args['table_display_mode'];
            }
            if (isset($args['mobile_layout_mode']) && !empty($args['mobile_layout_mode'])) {
                $arguments['mobileLayoutMode'] = $args['mobile_layout_mode'];
            }
            if (isset($args['page_limit']) && !empty($args['page_limit'])) {
                $arguments['numOfRecordsPerPage'] = strval($args['page_limit']);
            }
            if (isset($args['hide_table_header'])) {
                $arguments['hideTableHeader'] = $this->getBool($args['hide_table_header']);
            }
            if (isset($args['show_serial_number_column'])) {
                $arguments['serialNumberColumn'] = $this->getBool($args['show_serial_number_column']);
            }
            if (isset($args['search'])) {
                $arguments['desktop-search'] = $this->getBool($args['search']);
            }
            if (isset($args['sorting'])) {
                $arguments['desktop-sort'] = $this->getBool($args['sorting']);
            }
            if (isset($args['filters'])) {
                $arguments['desktop-filter'] = $this->getBool($args['filters']);
            }

            // error_log(' general : ' . print_r($general, true));
            // error_log(' arguments : ' . print_r($arguments, true));
            // error_log(' tableLevelDisplay : ' . print_r($tableLevelDisplay, true));

            $display_settings = array_replace_recursive($general, $tableLevelDisplay, $arguments);

            // error_log(' display_settings : ' . print_r($display_settings, true));

            return $display_settings;
        }

        public function getBool($value = false)
        {
            $boolean = false;

            if ($value == true || $value == 1 || $value == "true" || $value == "1") {
                $boolean = true;
            }

            return $boolean;
        }

        public function get_itemProps($collection = [])
        {
            $table_data['columns'] = $this->get_columns($this->columns, $collection);

            /** Getting Records */
            $table_instance = $this->tablesome_db->create_table_instance($this->table_id);
            $number = isset($collection['pagination']) && $collection['pagination'] == 1 ? $collection["display"]['numOfRecordsPerPage'] : 0;

            $query = $this->tablesome_db->query(array(
                'table_id' => $this->table_id,
                'table_name' => $table_instance->name,
                'number' => $number,
                'orderby' => array('rank_order', 'id'),
                'order' => 'asc',
            ));

            $records = isset($query->items) ? $query->items : [];

            $table_data["rows"] = $this->tablesome_db->get_formatted_rows($records, $this->table_meta, $collection);

            return $table_data;
        }

        protected function get_rows($table_data)
        {
            $processed_rows = [];
            $rows = isset($table_data['records']) && !empty($table_data['records']) ? $table_data['records'] : [];
            // Create New table
            if (empty($rows)) {
                array_push($processed_rows, [
                    "record_id" => 0,
                    "content" => [""],
                    "rank_order" => "0|100000:",
                ]);
            }

            $columns = isset($table_data['columns']) && !empty($table_data['columns']) ? $table_data['columns'] : [];

            foreach ($rows as $index => $row) {
                array_push($processed_rows, $this->get_row($row, $columns));
            }

            // unset the $rows after get the row informations
            unset($rows);

            return $processed_rows;
        }

        protected function get_row($row, $columns)
        {
            $cells = isset($row['content']) ? $row['content'] : [];

            if (empty($cells)) {
                return $row;
            }
            /** get exclude column ids */
            $exclude_column_ids = isset($this->collection['exclude_column_ids']) && !empty($this->collection['exclude_column_ids']) ? explode(",", $this->collection['exclude_column_ids']) : [];

            foreach ($cells as $cell_key => $cell_data) {

                /** remove the cell if that cell key (or) ID has found in exclude columns array */
                if (in_array($cell_key, $exclude_column_ids)) {
                    unset($row['content'][$cell_key]);
                    continue;
                }

                // $value = isset($cell_data["value"]) && !empty($cell_data["value"]) ? $cell_data["value"] : $cell_data;
                $cell = [
                    'type' => get_tablesome_cell_type($cell_key, $columns),
                    'html' => isset($cell_data) && isset($cell_data["html"]) && !empty($cell_data["html"]) ? $cell_data["html"] : $cell_data["value"],
                    'value' => isset($cell_data) && isset($cell_data["value"]) ? $cell_data["value"] : "",
                ];

                if (isset($cell_data["link"]) && !empty($cell_data["link"])) {
                    $cell = array_merge($cell, ["link" => $cell_data["link"]]);
                }

                if (isset($cell_data["linkText"]) && !empty($cell_data["linkText"])) {
                    $cell = array_merge($cell, ["linkText" => $cell_data["linkText"]]);
                }

                $cell = apply_filters("tablesome_get_cell_data", $cell);

                $row['content'][$cell_key] = $cell;
            }

            return $row;
        }

        public function get_columns(array $columns, array $collectionProps)
        {
            // $exclude_column_ids values received as string
            $exclude_column_ids = isset($collectionProps['exclude_column_ids']) && !empty($collectionProps['exclude_column_ids']) ? explode(",", $collectionProps['exclude_column_ids']) : [];

            foreach ($columns as $index => $column) {

                $columns[$index]["name"] = html_entity_decode($column["name"]);

                if (!empty($exclude_column_ids) && isset($column['id']) && in_array($column['id'], $exclude_column_ids)) {
                    /** remove the exclude columns */
                    unset($columns[$index]);
                }
            }
            // re-arranging the orders.
            $columns = array_values($columns);

            return $columns;
        }

        public function get_table_title($post_id)
        {
            $post_title = '';

            if (!empty($post_id)) {
                $post = get_post($post_id);
            }

            if (isset($post)) {
                $post_title = $post->post_title;
            }

            return $post_title;
        }

    }
}
