<?php

namespace Tablesome\Includes\Modules\Automation;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

use Tablesome\Includes\Modules\Automation\Integrations\Notion\Database as NotionDatabase;
use Tablesome\Includes\Modules\Automation\Integrations\Tablesome;
use \Tablesome\Includes\Modules\Automation\Integrations\Mailchimp;

if (!class_exists('\Tablesome\Includes\Modules\Automation\Actions')) {
    class Actions
    {
        public static $instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_actions();

            }
            return self::$instance;
        }

        public function load_actions()
        {
            Mailchimp::get_instance();
            Tablesome::get_instance();
            NotionDatabase::get_instance();
        }

        public function run_actions($args)
        {
            if ($this->is_valid_action($args["action_meta"])) {
                $actions = [
                    [
                        'action' => 'add_row',
                        "hook" => "tablesome_automation/add_record_to_tablesome_db",
                        "args" => $args,
                    ],

                    [
                        'action' => 'add_contact',
                        "hook" => "tablesome_automation/add_contact_to_mailchimp",
                        "args" => $args,
                    ],
                    [
                        'action' => 'add_page', // In Notion Database, they called it "Page" for adding records in the database.
                        "hook" => "tablesome_automation/add_page_to_notion_db",
                        "args" => $args,
                    ],
                ];

                foreach ($actions as $action) {
                    if ($this->is_valid_action_by_action_name($args["action_meta"]['action_id'], $action['action'])) {
                        /***
                         *  Check and modify the DB table columns if auto-detect option is enabled.
                         *  "Auto-Detect" fields option is only avail on Tablesome "Add Row" action.
                         * */
                        if ($action['action'] == 'add_row') {
                            do_action('tablesome_automation/auto_detect_option_enabled', $args);

                            // Store Exra Information
                            do_action('tablesome_automation/modified_smart_fields_columns', $args);

                        }

                        do_action($action['hook'], $action['args']);

                        /**
                         * Log the action
                         *
                         * @param mix $action_info
                         */
                        do_action('tablesome_automation/add_log', $action);
                    }
                }
            }

        }

        protected function is_valid_action($action_info)
        {
            // $match_columns = isset($action_info['match_columns']) ? $action_info['match_columns'] : [];
            $integration = isset($action_info['integration']) ? $action_info['integration'] : 'tablesome';
            $action_id = isset($action_info['action_id']) ? $action_info['action_id'] : 0;

            if (empty($action_id) || !$this->is_valid_action_by_integration($action_id, $integration)) {
                return false;
            }

            return true;
        }

        protected function is_valid_action_by_action_name($action_id, $action_name)
        {
            $is_valid = false;
            $actions = apply_filters('tablesome_automation/the_actions', array());
            foreach ($actions as $action_meta) {
                $dont_have_meta_actions = (!isset($action_meta['actions']) || empty($action_meta['actions)'])) ? true : false;
                if (empty($dont_have_meta_actions)) {
                    continue;
                }
                foreach ($action_meta['actions'] as $action_info) {
                    $is_valid = ($action_info['id'] == $action_id && $action_info['name'] == $action_name);
                    if ($is_valid) {
                        break;
                    }
                }
                if ($is_valid) {
                    break;
                }
            }
            return $is_valid;
        }

        private function is_valid_action_by_integration($action_id, $integration)
        {
            $is_valid = false;
            $action = $this->get_action_by_integration($integration);
            if (empty($action)) {
                return false;
            }

            foreach ($action['actions'] as $action_info) {
                $is_valid = ($action_info['id'] == $action_id);
                if ($is_valid) {
                    break;
                }
            }
            return $is_valid;
        }

        private function get_action_by_integration($integration)
        {
            $action = '';
            $actions = apply_filters('tablesome_automation/the_actions', array());
            foreach ($actions as $action_meta) {
                if ($action_meta['integration'] == $integration) {
                    $action = $action_meta;
                    break;
                }
            }
            return $action;
        }

        public function get_action_by_action_id($action_id)
        {
            $action = [];
            if (empty($action_id)) {
                return $action;
            }
            $actions = apply_filters('tablesome_automation/the_actions', array());
            foreach ($actions as $data) {
                foreach ($data['actions'] as $action_meta) {
                    if ($action_id == $action_meta['id']) {
                        $action = $action_meta;
                        break;
                    }
                }
            }

            return $action;
        }

        public function get_action_integration_name_by_id($action_id)
        {
            $name = '';
            if (empty($action_id)) {
                return $name;
            }
            $actions = apply_filters('tablesome_automation/the_actions', array());
            foreach ($actions as $data) {
                foreach ($data['actions'] as $action_meta) {
                    if ($action_id == $action_meta['id']) {
                        $name = $data['integration_label'];
                        break;
                    }
                }
            }

            return $name;
        }
    }
}