<?php

namespace Tablesome\Includes\Modules\Automation\Integrations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Integrations\Elementor')) {
    class Elementor
    {
        public $unsupported_formats = array(
            'hidden',
            'password',
            'upload',
            'acceptance',
            'step',
            'recaptcha',
            'recaptcha_v3',
            'honeypot',

            // can be used in the future
            'time',
            'html',
        );

        public static $instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_hooks();
                // $forms = self::$instance->get_form_collection([]);
                // error_log(' elementor forms : ' . print_r($forms, true));

            }

            return self::$instance;
        }

        public function load_hooks()
        {
            add_filter('tablesome_automation/the_triggers', array($this, 'get_triggers'), 10, 2);
            add_filter('tablesome_automation/get_form_collection', array($this, 'get_form_collection'), 10, 2);

            add_action("elementor_pro/forms/new_record", array($this, 'elementor_process_entry_save'), 10, 4);

        }

        public function get_triggers($triggers)
        {
            // $is_active = class_exists('Elementor\Plugin') ? true : false;
            $is_active = defined('ELEMENTOR_PRO_PATH') ? true : false;

            $triggers[] = array(
                'integration' => 'elementor',
                'integration_label' => __('Elementor', 'tablesome'),
                'trigger' => 'tablesome_elementor_form_submit',
                'trigger_id' => 3,
                'trigger_label' => __('On Form Submit', 'tablesome'),
                'is_active' => $is_active,
                'is_premium' => "no",
            );

            return $triggers;
        }

        public function elementor_process_entry_save($record, $handler)
        {
            if (empty($record)) {
                return;
            }

            $form_id = $record->get_form_settings('id');
            if (empty($form_id)) {
                return;
            }

            $submission_data = $this->get_formatted_posted_data($record->get('fields'));

            $action_data = array(
                'integration' => 'elementor',
                'form_submission_id' => $form_id,
                'form_submission_data' => $submission_data,
                'form_title' => $record->get('form_settings')['form_name'],
                'elementor_data' => array(
                    'fields' => $record->get('fields'),
                ),
            );

            /**
             * Trigger action when submitting a from
             */
            do_action('tablesome_automation/trigger', $action_data);

        }

        public function get_form_collection($collection)
        {
            $forms = $this->get_forms();
            if (empty($forms)) {
                return $collection;
            }

            foreach ($forms as $form) {
                $form['fields'] = $this->get_form_fields($form["form_id"]);
                $collection[] = $form;
            }
            return $collection;
        }

        public function get_forms()
        {
            $forms = [];
            global $wpdb;
            $post_metas = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT pm.meta_value
                        FROM $wpdb->postmeta pm
                            LEFT JOIN $wpdb->posts p
                                ON p.ID = pm.post_id
                        WHERE p.post_type IS NOT NULL
                        AND p.post_status = %s
                        AND pm.meta_key = %s
                        AND pm.`meta_value` LIKE %s",
                    'publish',
                    '_elementor_data',
                    '%%form_fields%%'
                )
            );

            if (!empty($post_metas)) {
                foreach ($post_metas as $post_meta) {
                    $inner_forms = self::get_all_inner_forms(json_decode($post_meta->meta_value));
                    if (!empty($inner_forms)) {
                        foreach ($inner_forms as $form) {
                            $forms[] = array(
                                'form_id' => $form->id,
                                'form_title' => $form->settings->form_name . " (ID: " . $form->id . ")",
                                'form_type' => 'elementor',
                            );
                        }
                    }
                }
            }

            return $forms;
        }

        /**
         * Return all the specific fields of a form ID
         */
        public function get_form_fields($form_id)
        {
            $fields = [];

            global $wpdb;
            $query = "SELECT ms.meta_value  FROM {$wpdb->postmeta} ms JOIN {$wpdb->posts} p on p.ID = ms.post_id WHERE ms.meta_key LIKE '_elementor_data' AND ms.meta_value LIKE '%form_fields%' AND p.post_status = 'publish' ";
            $post_metas = $wpdb->get_results($query);

            if (!empty($post_metas)) {
                foreach ($post_metas as $post_meta) {
                    $inner_forms = self::get_all_inner_forms(json_decode($post_meta->meta_value));
                    if (!empty($inner_forms)) {
                        foreach ($inner_forms as $form) {
                            if ($form->id == $form_id) {
                                if (!empty($form->settings->form_fields)) {
                                    foreach ($form->settings->form_fields as $field) {
                                        // error_log(' field : ' . print_r($field, true));

                                        $type = isset($field->field_type) && !empty($field->field_type) ? $field->field_type : 'text';
                                        if (!in_array($type, $this->unsupported_formats)) {
                                            $options = self::get_options($field);
                                            $single_field = [
                                                'id' => $field->custom_id,
                                                'name' => !empty($field->field_label) ? $field->field_label : 'unknown',

                                            ];
                                            if (!empty($options)) {
                                                $single_field['options'] = $options;
                                            }

                                            $fields[] = $single_field;
                                        }

                                    }
                                }
                            }
                        }
                    }
                }
            }

            // error_log(' fields : ' . print_r($fields, true));

            return $fields;
        }

        public static function get_all_inner_forms($elements)
        {
            $block_is_on_page = array();
            if (!empty($elements)) {
                foreach ($elements as $element) {
                    if ('widget' === $element->elType && 'form' === $element->widgetType) {
                        $block_is_on_page[] = $element;
                    }
                    if (!empty($element->elements)) {
                        $inner_block_is_on_page = self::get_all_inner_forms($element->elements);
                        if (!empty($inner_block_is_on_page)) {
                            $block_is_on_page = array_merge($block_is_on_page, $inner_block_is_on_page);
                        }
                    }
                }
            }

            return $block_is_on_page;
        }

        public static function get_options($field)
        {
            $options = [];
            if (isset($field->field_options) && !empty($field->field_options)) {
                $options_list = preg_split('/\r\n|\r|\n/', $field->field_options);

                foreach ($options_list as $option) {
                    array_push($options, [
                        'label' => $option,
                        'value' => $option,
                    ]);
                }
            }

            return $options;
        }

        public function get_formatted_posted_data($fields)
        {
            $data = array();
            if (empty($fields)) {
                return $data;
            }
            foreach ($fields as $id => $field) {
                $value = $field['value'];
                if ($field['type'] == 'date' && is_valid_tablesome_date($value, 'Y-m-d')) {
                    $unix_timestamp = convert_tablesome_date_to_unix_timestamp($value, 'Y-m-d');
                    $unix_timestamp = $unix_timestamp * 1000; // convert to milliseconds
                }
                $data[$id] = array(
                    'value' => $value,
                    'type' => $field['type'],
                    'unix_timestamp' => isset($unix_timestamp) ? $unix_timestamp : '', // use this prop when the column format type is date
                );
            }
            return $data;
        }
    }
}
