<?php

namespace Tablesome\Includes\Modules\Automation\Integrations;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
// Exit if accessed directly
if ( !class_exists( '\\Tablesome\\Includes\\Modules\\Automation\\Integrations\\Mailchimp' ) ) {
    class Mailchimp
    {
        public  $api_key = '' ;
        public  $api_status = false ;
        public static  $instance = null ;
        public static function get_instance()
        {
            
            if ( is_null( self::$instance ) ) {
                self::$instance = new self();
                self::$instance->load_hooks();
                $api_key = get_option( TABLESOME_MAILCHIMP_API_KEY );
                $api_status = get_option( TABLESOME_MAILCHIMP_API_STATUS );
                self::$instance->api_key = ( $api_key ? $api_key : self::$instance->api_key );
                self::$instance->api_status = ( $api_status ? $api_status : self::$instance->api_status );
            }
            
            return self::$instance;
        }
        
        public function load_hooks()
        {
            add_filter(
                'tablesome_automation/the_actions',
                array( self::$instance, 'get_actions' ),
                10,
                2
            );
            add_filter(
                'tablesome_automation/get_mailchimp_collection',
                array( self::$instance, 'get_collection' ),
                10,
                2
            );
            add_action(
                'tablesome_automation/add_contact_to_mailchimp',
                [ self::$instance, 'add_contact_to_mailchimp' ],
                10,
                2
            );
        }
        
        public function add_api( $api_key )
        {
            $this->api_key = $api_key;
            update_option( TABLESOME_MAILCHIMP_API_KEY, $api_key );
        }
        
        public function remove_api_data()
        {
            delete_option( TABLESOME_MAILCHIMP_API_KEY );
            delete_option( TABLESOME_MAILCHIMP_API_STATUS );
            delete_option( TABLESOME_MAILCHIMP_API_STATUS_MESSAGE );
        }
        
        public function get_actions( $actions )
        {
            $actions[] = array(
                'integration'       => 'mailchimp',
                'integration_label' => __( 'Mailchimp', 'tablesome' ),
                'is_active'         => $this->api_status,
                'is_premium'        => false,
                'actions'           => array( array(
                'id'    => 2,
                'name'  => 'add_contact',
                'label' => __( 'Add Contact', 'tablesome' ),
            ) ),
            );
            return $actions;
        }
        
        /* Get Mailchimp datacenter value from the API Key
         *
         * Ref :- https://mailchimp.com/developer/marketing/docs/fundamentals/#the-basics
         * @return string
         */
        public function get_dc_value_from_api_key()
        {
            $dc = substr( $this->api_key, strpos( $this->api_key, '-' ) + 1 );
            return ( !empty($dc) ? $dc : '' );
        }
        
        public function get_api_base_path()
        {
            $dc = $this->get_dc_value_from_api_key();
            return 'https://' . $dc . '.api.mailchimp.com/3.0';
        }
        
        /**
         * Use of the below method for checking API status is active or not.
         *
         */
        public function ping()
        {
            $args = array(
                'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( 'tablesome:' . $this->api_key ),
            ),
            );
            $url = $this->get_api_base_path() . '/ping';
            $response = wp_remote_get( $url, $args );
            $data = json_decode( wp_remote_retrieve_body( $response ), true );
            $status = false;
            
            if ( wp_remote_retrieve_response_code( $response ) != 200 ) {
                $message = ( isset( $data['detail'] ) ? $data['detail'] : 'The API key is invalid.' );
                update_option( TABLESOME_MAILCHIMP_API_STATUS, $status );
                update_option( TABLESOME_MAILCHIMP_API_STATUS_MESSAGE, $message );
                return array(
                    'status'  => $status,
                    'message' => $message,
                );
            }
            
            $health_status = ( isset( $data['health_status'] ) ? $data['health_status'] : $status );
            $status = ( !empty($health_status) && $health_status === "Everything's Chimpy!" ? true : false );
            $message = ( $status == true ? 'Connected' : 'Not Connected' );
            update_option( TABLESOME_MAILCHIMP_API_STATUS, $status );
            update_option( TABLESOME_MAILCHIMP_API_STATUS_MESSAGE, $message );
            return array(
                'status'  => $status,
                'message' => $message,
            );
        }
        
        /**
         * Add (or) Update the member to the list
         * API Document URL https://mailchimp.com/developer/marketing/api/list-members/add-or-update-list-member/
         * @param array $subscriber_data
         *
         */
        public function add_contact_to_the_lists( $action_meta, $subscriber_data )
        {
            if ( isset( $this->api_key ) && empty($this->api_key) ) {
                return;
            }
            $email_address = ( isset( $subscriber_data['email_address'] ) ? $subscriber_data['email_address'] : '' );
            // Return if empty
            if ( empty($email_address) || !$this->is_valid_email( $email_address ) ) {
                return;
            }
            $subscriber_data = $this->get_subscriber_data( $subscriber_data );
            $list_id = $action_meta['list_id'];
            $subscriber_hash = md5( strtolower( $email_address ) );
            $url = $this->get_api_base_path() . '/lists/' . $list_id . '/members/' . $subscriber_hash . '?skip_merge_validation=true';
            $args = array(
                'method'  => 'PUT',
                'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( 'tablesome:' . $this->api_key ),
            ),
                'body'    => json_encode( $subscriber_data ),
            );
            $response = wp_remote_post( $url, $args );
            $data = json_decode( $response['body'], true );
            if ( $response['response']['code'] != 200 ) {
                /** The subscriber doesn't add to the list */
                // error_log('$response : ' . print_r($response, true));
                // TODO: Should add log for the subscriber doesn't add.
                return;
            }
            //TODO: Should add log for event successfully done.
            // error_log('$data : ' . print_r($data, true));
            return;
        }
        
        public function get_subscriber_data( $data )
        {
            /**
             * Ref Url's
             * https://mailchimp.com/developer/marketing/docs/merge-fields/#structure
             * https://mailchimp.com/developer/marketing/docs/merge-fields/#add-merge-data-to-contacts
             */
            $available_address_tag_fields = array_column( $this->get_default_address_fields(), 'id' );
            $subscriber_data = array(
                'status'        => 'subscribed',
                'email_address' => $data['email_address'],
            );
            foreach ( $data as $tag => $value ) {
                $is_address_field_props = in_array( $tag, $available_address_tag_fields );
                if ( $is_address_field_props ) {
                    $subscriber_data['ADDRESS1'][$tag] = $value;
                }
                if ( !$is_address_field_props && $tag != 'email_address' ) {
                    $subscriber_data['merge_fields'][$tag] = $value;
                }
            }
            return $subscriber_data;
        }
        
        public function get_lists()
        {
            $lists = array();
            $status = get_option( TABLESOME_MAILCHIMP_API_STATUS );
            if ( !$status ) {
                return $lists;
            }
            $args = array(
                'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( 'tablesome:' . $this->api_key ),
            ),
            );
            $url = $this->get_api_base_path() . '/lists?count=1000';
            $response = wp_remote_get( $url, $args );
            $data = json_decode( wp_remote_retrieve_body( $response ), true );
            if ( wp_remote_retrieve_response_code( $response ) != 200 ) {
                // error_log('$details : ' . print_r($data->detail, true));
                return $lists;
            }
            $lists_info = ( isset( $data['lists'] ) ? $data['lists'] : [] );
            if ( empty($lists_info) ) {
                return $lists;
            }
            foreach ( $lists_info as $list_info ) {
                $lists[] = array(
                    'id'    => $list_info['id'],
                    'value' => $list_info['name'],
                );
            }
            return $lists;
        }
        
        public function get_all_tags_from_audience( $audience_id )
        {
            $tags = array();
            if ( isset( $this->api_key ) && empty($this->api_key) ) {
                return $tags;
            }
            //TODO: The below endpoint is official endpoint for getting the tags from audience, but it didn't work
            $url = $this->get_api_base_path() . '/tags/lists';
            /***
             * // TODO: Using the endpoint for searching tags to the selected audience
             *  If the search key is empty then it's return the all tags. So, used on this endpoint instead of above.
             */
            $url = $this->get_api_base_path() . "/lists/{$audience_id}/tag-search";
            $payload = array(
                'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( 'tablesome:' . $this->api_key ),
            ),
            );
            $response = wp_remote_get( $url, $payload );
            if ( is_wp_error( $response ) ) {
                return array();
            }
            if ( $response['response']['code'] != 200 ) {
                return $tags;
            }
            $data = json_decode( wp_remote_retrieve_body( $response ), true );
            $tags = ( isset( $data['tags'] ) && !empty($data['tags']) ? $data['tags'] : [] );
            return $tags;
        }
        
        public function get_collection( $collection )
        {
            /** Get all mailchimp audiences */
            $audiences = $this->get_lists();
            /** add audience extra props  */
            $audiences = $this->add_audience_props( $audiences );
            $status = get_option( TABLESOME_MAILCHIMP_API_STATUS );
            $message = get_option( TABLESOME_MAILCHIMP_API_STATUS_MESSAGE );
            $api_not_configured = empty($status) && empty($message);
            if ( $api_not_configured ) {
                $message = 'Please configure Mailchimp API in Tablesome for this action to work.';
            }
            return array(
                'audiences' => $audiences,
                'api'       => array(
                'status'       => $status,
                'message'      => $message,
                'redirect_url' => admin_url( 'edit.php?post_type=' . TABLESOME_CPT . '&page=tablesome-settings#tab=integrations/mailchimp' ),
            ),
            );
        }
        
        public function add_audience_props( $audiences )
        {
            if ( empty($audiences) ) {
                return [];
            }
            foreach ( $audiences as $index => $audience ) {
                $audiences[$index]['tags'] = $this->get_all_tags_from_audience( $audience['id'] );
                $audiences[$index]['fields'] = $this->get_all_fields_from_audience( $audience );
            }
            return $audiences;
        }
        
        public function get_all_fields_from_audience( $audience )
        {
            $fields = [];
            $url = $this->get_api_base_path() . "/lists/{$audience['id']}/merge-fields?count=1000";
            $payload = array(
                'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( 'tablesome:' . $this->api_key ),
            ),
            );
            $response = wp_remote_get( $url, $payload );
            $data = json_decode( wp_remote_retrieve_body( $response ), true );
            if ( $response['response']['code'] != 200 ) {
                return $fields;
            }
            $merge_fields = ( isset( $data['merge_fields'] ) ? $data['merge_fields'] : [] );
            if ( empty($merge_fields) ) {
                return $fields;
            }
            /** Ordering the fields based on display order value  */
            $display_orders = array_column( $merge_fields, 'display_order' );
            // Ref:- https://www.php.net/manual/en/function.array-multisort.php
            array_multisort( $display_orders, SORT_ASC, $merge_fields );
            /***
             * Important:- Manually, add the email-address field if doesn't exist in the audience fields.
             * As per doc, we can't add a contact without subscriber email-address.
             */
            $email_address_exists = in_array( 'email_address', array_column( $merge_fields, 'tag' ) );
            if ( !$email_address_exists ) {
                $fields[] = array(
                    'id'    => 'email_address',
                    'label' => __( 'Email Address', 'tablesome' ),
                );
            }
            foreach ( $merge_fields as $field ) {
                $type = $field['type'];
                $tag = $field['tag'];
                /**
                 * Check the address field from the below url
                 * @see https://mailchimp.com/developer/marketing/docs/merge-fields/#add-merge-data-to-contacts
                 * The address is one type of field in Mailchimp. This field has a collection of properties. like street, city, country, Pincode..
                 * Those props couldn't get from API. It's added manually.
                 */
                
                if ( $type == 'address' ) {
                    $address_fields = $this->get_default_address_fields();
                    $fields = array_merge( $fields, $address_fields );
                } else {
                    $fields[] = array(
                        'id'    => $tag,
                        'label' => $field['name'],
                    );
                }
            
            }
            return $fields;
        }
        
        public function get_default_address_fields()
        {
            /**
             * @see https://mailchimp.com/developer/marketing/docs/merge-fields/#add-merge-data-to-contacts
             */
            return array(
                array(
                'id'    => 'addr1',
                'label' => __( 'Street Address', 'tablesome' ),
            ),
                array(
                'id'    => 'addr2',
                'label' => __( 'Address Line 2', 'tablesome' ),
            ),
                array(
                'id'    => 'city',
                'label' => __( 'City', 'tablesome' ),
            ),
                array(
                'id'    => 'state',
                'label' => __( 'State/Prov/Region', 'tablesome' ),
            ),
                array(
                'id'    => 'zip',
                'label' => __( 'Postal/Zip', 'tablesome' ),
            ),
                array(
                'id'    => 'country',
                'label' => __( 'Country', 'tablesome' ),
            )
            );
        }
        
        public function get_tag_names_by_ids( $mailchimp_meta_data )
        {
            $selected_tags = ( isset( $mailchimp_meta_data['tags'] ) ? $mailchimp_meta_data['tags'] : [] );
            $names = array();
            if ( empty($selected_tags) ) {
                return $names;
            }
            $all_tags = $this->get_all_tags_from_audience( $mailchimp_meta_data['list_id'] );
            if ( empty($all_tags) ) {
                return $names;
            }
            foreach ( $all_tags as $tag ) {
                $id = $tag['id'];
                if ( in_array( $id, $selected_tags ) ) {
                    $names[] = $tag['name'];
                }
            }
            return $names;
        }
        
        public function is_valid_email( $email_address )
        {
            if ( !filter_var( $email_address, FILTER_VALIDATE_EMAIL ) ) {
                return false;
            }
            return true;
        }
        
        public function can_add_contact_to_list( $action_meta, $form_submitted_data )
        {
            $list_id = ( isset( $action_meta['list_id'] ) ? $action_meta['list_id'] : '' );
            if ( empty($list_id) ) {
                return false;
            }
            $match_fields = ( isset( $action_meta['match_fields'] ) ? $action_meta['match_fields'] : [] );
            if ( empty($match_fields) ) {
                return false;
            }
            $email_address_configured = $this->is_email_address_configured( $match_fields );
            if ( !$email_address_configured ) {
                return false;
            }
            return true;
        }
        
        public function check_defined_conditions_are_valid( $conditions, $form_submitted_data )
        {
            $form_data = $form_submitted_data['form_submission_data'];
            $form_submitted_by = ( isset( $form_submitted_data['integration'] ) ? $form_submitted_data['integration'] : '' );
            $is_valid = true;
            if ( empty($conditions) ) {
                return true;
            }
            $conditional_statement_is_true = true;
            foreach ( $conditions as $condition ) {
                $field = ( isset( $condition['field'] ) ? $condition['field'] : '' );
                $operator = ( isset( $condition['operator'] ) ? $condition['operator'] : '' );
                $value = ( isset( $condition['value'] ) ? $condition['value'] : '' );
                if ( empty($field) || empty($operator) ) {
                    continue;
                }
                $submitted_value = ( isset( $form_data[$field]['value'] ) ? $form_data[$field]['value'] : '' );
                $to_be_continued = !isset( $form_data[$field]['value'] ) || !$conditional_statement_is_true;
                
                if ( $to_be_continued ) {
                    $is_valid = false;
                    continue;
                }
                
                
                if ( $form_submitted_by == 'wpforms' && !empty($value) && intval( $value ) ) {
                    $props = array(
                        'field'           => $field,
                        'value'           => $value,
                        'submitted_value' => $submitted_value,
                    );
                    $submitted_value = apply_filters( 'tablesome_automation/get_wpform_field_option_id_by_value', $props, $form_submitted_data );
                }
                
                
                if ( $operator == 'equal_to' ) {
                    $is_valid = $value == $submitted_value;
                } else {
                    if ( $operator == 'not_equal_to' ) {
                        $is_valid = $value != $submitted_value;
                    }
                }
                
                $conditional_statement_is_true = $is_valid;
            }
            return $is_valid;
        }
        
        public function is_email_address_configured( $match_fields )
        {
            $is_configured = false;
            foreach ( $match_fields as $match_field ) {
                $type_name = ( isset( $match_field['type_name'] ) ? $match_field['type_name'] : '' );
                $field_name = ( isset( $match_field['field_name'] ) ? $match_field['field_name'] : '' );
                $is_configured = $type_name == 'email_address' && $field_name;
                if ( $is_configured ) {
                    break;
                }
            }
            return $is_configured;
        }
        
        public function get_subscriber_data_from_submitted_form( $action_meta, $form_submitted_data )
        {
            $data = array();
            $match_fields = ( isset( $action_meta['match_fields'] ) ? $action_meta['match_fields'] : [] );
            $form_data = ( isset( $form_submitted_data['form_submission_data'] ) ? $form_submitted_data['form_submission_data'] : [] );
            foreach ( $match_fields as $match_field ) {
                $type_name = ( isset( $match_field['type_name'] ) ? $match_field['type_name'] : '' );
                $field_name = ( isset( $match_field['field_name'] ) ? $match_field['field_name'] : '' );
                if ( empty($type_name) || empty($field_name) ) {
                    continue;
                }
                $value = ( isset( $form_data[$field_name]['value'] ) ? $form_data[$field_name]['value'] : '' );
                $data[$type_name] = $value;
            }
            return $data;
        }
        
        public function add_contact_to_mailchimp( $args )
        {
            $action_meta = $args["action_meta"];
            $form_submitted_data = $args["form_submitted_data"];
            $can_add_contact_to_list = $this->can_add_contact_to_list( $action_meta, $form_submitted_data );
            if ( !$can_add_contact_to_list ) {
                return;
            }
            $subscriber_data = $this->get_subscriber_data_from_submitted_form( $action_meta, $form_submitted_data );
            $this->add_contact_to_the_lists( $action_meta, $subscriber_data );
        }
    
    }
}