<?php

namespace Tablesome\Includes\Modules\Automation\Integrations\Notion;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

trait Properties
{

    public function get_property($property_id, $database)
    {
        $data = [];
        $properties = isset($database['properties']) ? $database['properties'] : array();
        if (empty($properties)) {
            return $data;
        }
        foreach ($properties as $property) {
            if ($property['id'] == $property_id) {
                $data = $property;
                break;
            }
        }
        return $data;
    }

    public function get_property_values($property, $value)
    {
        $method_name = "get_{$property['type']}_values";
        if (method_exists($this, $method_name)) {
            return $this->$method_name($property, $value);
        } else {
            return [];
        }
    }

    public function get_title_values($property, $value)
    {
        $data = [];
        $data['title'] = array(
            array(
                'type' => 'text',
                'text' => array(
                    'content' => $value,
                ),
            ),
        );
        return $data;
    }

    public function get_rich_text_values($property, $value)
    {
        $data = [];
        $data['rich_text'] = array(
            array(
                'type' => "text",
                "text" => array(
                    'content' => $value,
                ),
            ),
        );
        return $data;
    }

    public function get_number_values($property, $value)
    {
        $data = null;
        $float_val = (float) $value;
        $int_val = (int) $value;
        $double_val = (double) $value;

        if (is_float($float_val)) {
            $data = floatval($float_val);
        } else if (is_int($int_val)) {
            $data = intval($int_val);
        } else if (is_double($double_val)) {
            $data = doubleval($double_val);
        }

        return array(
            'number' => $data,
        );
    }

    public function get_select_values($property, $value)
    {
        return array(
            'select' => array(
                'name' => $value,
            ),
        );
    }

    public function get_multi_select_values($property, $value)
    {
        $values = isset($value) && !empty($value) ? explode(',', $value) : [];
        if (empty($values)) {
            return [];
        }

        $data = [];
        $data['multi_select'] = array();
        foreach ($values as $value) {
            $data['multi_select'][] = array(
                'name' => $value,
            );
        }
        return $data;
    }

    public function get_checkbox_values($property, $value)
    {
        return array(
            'checkbox' => $value,
        );
    }

    public function get_url_values($property, $value)
    {
        return array(
            'url' => $value,
        );
    }

    public function get_email_values($property, $value)
    {
        return array(
            'email' => $value,
        );
    }

    public function get_phone_number_values($property, $value)
    {
        return array(
            'phone_number' => $value,
        );
    }

    // public function get_people_values($property, $value)
    // {
    // }

    public function get_date_values($property, $value)
    {
        if (empty($value) && !is_valid_tablesome_date($value, 'Y-m-d')) {
            return [];
        }

        $datetime = new \DateTime($value);
        $date = $datetime->format('Y-m-d');

        return array(
            'date' => array(
                'start' => $date,
            ),
        );
    }

}