<?php

namespace Tablesome\Includes\Modules\Automation\Integrations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Integrations\Tablesome')) {
    class Tablesome
    {
        public static $instance = null;

        /*** Below defined form fields are inserted in form by 3-rd party plugins */
        public $un_supported_fields = array(
            'vx_width',
            'vx_height',
            'vx_url',
        );

        public $column_formats = array(
            'textarea' => array('textarea', 'address'),
            'date' => array('date', 'date-time'),
            'email' => array('email'),
            'url' => array('url'),
            'number' => array('number-slider', 'rating', 'number', 'postdata', 'currency', 'calculation'),
        );

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_hooks();
            }

            return self::$instance;
        }

        public function load_hooks()
        {
            add_filter('tablesome_automation/the_actions', [self::$instance, 'get_actions'], 10, 2);
            add_action('tablesome_automation/add_record_to_tablesome_db', [self::$instance, 'add_record_to_tablesome_db'], 10, 2);

            add_action('tablesome_automation/auto_detect_option_enabled', [self::$instance, 'auto_detect_option_enabled'], 10, 2);
            add_action('tablesome_automation/modified_smart_fields_columns', [self::$instance, 'modified_smart_fields_columns'], 10, 2);

        }

        public function get_actions($actions)
        {
            $actions[] = array(
                'integration' => 'tablesome',
                'integration_label' => __('Tablesome', 'tablesome'),
                'is_active' => true,
                'is_premium' => false,
                'actions' => array(
                    array(
                        'id' => 1,
                        'name' => 'add_row',
                        'label' => __('Add Row', 'tablesome'),
                    ),
                ),
            );
            return $actions;
        }

        public function add_record_to_tablesome_db($args)
        {
            $table = $args['table'];
            $trigger_position = $args['trigger_position'];
            $action_position = $args['action_position'];

            $table_meta = get_tablesome_data($table->ID);
            $triggers_meta = get_tablesome_table_triggers($table->ID);

            $action_meta = isset($triggers_meta[$trigger_position]['actions'][$action_position]) ? $triggers_meta[$trigger_position]['actions'][$action_position] : array();
            $form_submitted_data = isset($args['form_submitted_data']) ? $args['form_submitted_data'] : [];

            if (empty($action_meta) || empty($table_meta) || empty($form_submitted_data)) {
                return;
            }

            $form_values = $this->get_form_values_by_matching_columns($action_meta, $table_meta, $form_submitted_data);
            if (empty($form_values)) {
                return;
            }

            $tablesome_db = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();

            $db_table = $tablesome_db->create_table_instance($table->ID, $table_meta);
            $query = $tablesome_db->query(array(
                'table_id' => $table->ID,
                'table_name' => $db_table->name,
            ));

            $default_record_values = $this->get_default_insert_record_values($table->ID);
            $insert_record_data = array_merge(array(), $default_record_values, $form_values);
            $tablesome_db->insert($query, $insert_record_data);
        }

        public function get_default_insert_record_values($table_id)
        {
            $author_id = get_current_user_id();

            return array(
                'post_id' => $table_id,
                'author_id' => $author_id,
                'created_at' => current_time('mysql', true),
                'updated_at' => current_time('mysql', true),
                'rank_order' => '',
            );
        }

        public function get_form_values_by_matching_columns($action_meta, $table_meta, $data)
        {
            $form_values = array();
            $match_columns = isset($action_meta['match_columns']) ? $action_meta['match_columns'] : [];
            $is_autodetect_enabled = isset($action_meta['autodetect_enabled']) && !empty($action_meta['autodetect_enabled']) ? $action_meta['autodetect_enabled'] : false;
            $is_matchcolumns_enabled = isset($action_meta['matchcolumns_enabled']) && !empty($action_meta['matchcolumns_enabled']) ? $action_meta['matchcolumns_enabled'] : false;

            if (empty($match_columns)) {
                return $form_values;
            }
            $modes = array_column($match_columns, 'detection_mode');
            $have_manual_fields = (isset($modes) && count($modes) > 0 && in_array('manual', $modes));
            $table_columns = isset($table_meta['columns']) ? $table_meta['columns'] : [];

            foreach ($match_columns as $match_column_info) {

                $field_type = isset($match_column_info['field_type']) ? $match_column_info['field_type'] : '';
                if ($field_type == 'tablesome_smart_fields') {
                    continue;
                }

                /** Skip the iteration if the field-name or column-id prop doesn't exist in the match column array */
                if (!isset($match_column_info['field_name']) || !isset($match_column_info['column_id'])) {
                    continue;
                }

                $field_name = $match_column_info['field_name'];
                $column_id = $match_column_info['column_id'];
                $detection_mode = isset($match_column_info['detection_mode']) ? $match_column_info['detection_mode'] : 'auto';

                /** Skip the iteration when the autodetect option is un-checked and the detection_mode is "auto".  */
                if (!$is_autodetect_enabled && $detection_mode == 'auto') {
                    continue;
                }

                // if ($is_autodetect_enabled && !$is_matchcolumns_enabled && $detection_mode == 'manual') {
                //     continue;
                // }

                /**
                 *  Don't store the field value in auto-detected column if that field already have configure in other column in manually.
                 *  When the Manual matchcolumn option is ON and auto-detect enabled.
                 *   */
                if ($is_matchcolumns_enabled && $is_autodetect_enabled && $detection_mode == 'auto' && $have_manual_fields) {
                    $field_matched_in_manually = $this->the_field_matched_in_manually($field_name, $match_columns);
                    if ($field_matched_in_manually) {
                        continue;
                    }
                }

                $column_id_exists = $this->column_id_exists_in_table_meta($column_id, $table_meta);
                if (!$column_id_exists) {
                    continue;
                }
                $column_format = get_tablesome_cell_type($column_id, $table_columns);

                $field_value = isset($data['form_submission_data'][$field_name]['value']) ? $data['form_submission_data'][$field_name]['value'] : '';
                if ($column_format == 'date') {
                    $field_value = isset($data['form_submission_data'][$field_name]['unix_timestamp']) ? $data['form_submission_data'][$field_name]['unix_timestamp'] : '';
                }

                $db_column = "column_{$column_id}";
                $form_values[$db_column] = $field_value;
            }

            $smart_fields_values = $this->get_smart_fields_values();

            foreach ($match_columns as $match_column_info) {
                $field_type = isset($match_column_info['field_type']) ? $match_column_info['field_type'] : '';
                $column_id = isset($match_column_info['column_id']) ? intval($match_column_info['column_id']) : '';
                $field_name = isset($match_column_info['field_name']) ? $match_column_info['field_name'] : '';
                $detection_mode = isset($match_column_info['detection_mode']) ? $match_column_info['detection_mode'] : '';

                if ($field_type != 'tablesome_smart_fields' || !$column_id || !$field_name || $detection_mode != 'enabled') {
                    continue;
                }

                $db_column = "column_{$column_id}";
                $field_value = isset($smart_fields_values[$field_name]) ? $smart_fields_values[$field_name] : '';
                $form_values[$db_column] = $field_value;
            }

            return $form_values;
        }

        public function column_id_exists_in_table_meta($column_id, $table_meta)
        {
            $exists = false;
            if (!isset($table_meta['columns']) || empty($table_meta['columns'])) {
                return $exists;
            }

            foreach ($table_meta['columns'] as $column) {
                if ($column_id == $column['id']) {
                    $exists = true;
                    break;
                }
            }
            return $exists;
        }

        public function auto_detect_option_enabled($args)
        {
            $table = $args['table'];
            $table_id = isset($table->ID) ? $table->ID : 0;
            // return if the table id is 0 (or) missing the trigger & action prop
            if (empty($table) || !isset($args['trigger_position']) || !isset($args['action_position'])) {
                return;
            }

            // Get the table triggers meta
            $triggers_meta = get_tablesome_table_triggers($table_id);
            $is_autodetect_enabled = isset($args['action_meta']['autodetect_enabled']) && !empty($args['action_meta']['autodetect_enabled']) ? $args['action_meta']['autodetect_enabled'] : false;

            if (!$is_autodetect_enabled) {
                return;
            }
            $args['triggers_meta'] = $triggers_meta;

            $table_meta = get_tablesome_data($table_id);
            $trigger_meta = $args['trigger_meta'];
            $last_column_id = $table_meta['meta']['last_column_id'];

            // Get the current trigger & action position
            $action_position = intval($args['action_position']);
            $trigger_position = intval($args['trigger_position']);

            $matched_colums = isset($args['action_meta']['match_columns']) && !empty($args['action_meta']['match_columns']) ? $args['action_meta']['match_columns'] : [];

            // Collect modified fields from the submitted form
            $modified_fields = $this->get_modified_fields_from_submitted_form($matched_colums, $args);

            // return if the modified fields are empty
            if (empty($modified_fields)) {
                return;
            }

            foreach ($modified_fields as $modified_field) {

                /**
                 * If the field is matched then the method their column-id else it's return false.
                 */
                $field_matched_in_other_triggers = $this->the_field_matched_in_other_triggers($modified_field['name'], $args);
                $field_type = $modified_field['type'];

                if (!$field_matched_in_other_triggers) {
                    $last_column_id = $last_column_id + 1;

                    // Add New columns to the table meta
                    $table_meta['columns'][] = array(
                        'id' => $last_column_id,
                        'name' => $modified_field['label'],
                        'format' => $this->get_column_format_by_field_type($field_type),
                    );

                }

                // Add new matching columns to the trigger meta
                $trigger_meta['actions'][$action_position]['match_columns'][] = array(
                    'column_id' => $field_matched_in_other_triggers ? $field_matched_in_other_triggers : $last_column_id,
                    'field_name' => strval($modified_field['name']), // Convert the field value to string type if it's in integer type.
                    'detection_mode' => 'auto',
                );
            }

            // Update the last-column-id value to the table meta
            $table_meta['meta']['last_column_id'] = $last_column_id;

            if (!isset($triggers_meta[$trigger_position])) {
                return;
            }

            // Update the modified table-postmeta data.
            set_tablesome_data($table_id, $table_meta);
            // Store new fields in DB.
            $helper = new \Tablesome\Includes\Lib\Table_Crud_WP\Helper();
            $table_columns = $helper->get_table_columns($table_meta);

            $db_table = new \Tablesome_Table(array(
                'table_name' => "tablesome_table_{$table_id}",
            ));
            $db_table->modify_the_table($table_meta, $table_columns, []);

            // Update the trigger meta.
            $triggers_meta[$trigger_position] = []; // reset the trigger meta by trigger position
            $triggers_meta[$trigger_position] = $trigger_meta; // update the trigger meta by trigger position
            set_tablesome_table_triggers($table_id, $triggers_meta); // update the triggers meta in DB
        }

        public function get_modified_fields_from_submitted_form($match_columns, $args)
        {
            $modified_fields = [];
            $form_submission_data = isset($args['form_submitted_data']['form_submission_data']) && !empty($args['form_submitted_data']['form_submission_data']) ? $args['form_submitted_data']['form_submission_data'] : [];
            $integration = isset($args['form_submitted_data']['integration']) && !empty($args['form_submitted_data']['integration']) ? $args['form_submitted_data']['integration'] : '';
            $wpforms_fields_data = isset($args['form_submitted_data']['wpforms_data']) && !empty($args['form_submitted_data']['wpforms_data']['fields']) ? $args['form_submitted_data']['wpforms_data']['fields'] : [];
            $elementor_fields_data = isset($args['form_submitted_data']['elementor_data']) && !empty($args['form_submitted_data']['elementor_data']['fields']) ? $args['form_submitted_data']['elementor_data']['fields'] : [];

            foreach ($form_submission_data as $field_name => $field) {
                $field_name_exists = $this->field_name_exists_in_match_columns($field_name, $match_columns, $integration);

                if (!$field_name_exists) {

                    $label = $field_name;
                    if ($integration == 'wpforms') {
                        $label = $this->get_field_label_from_wpform($field_name, $wpforms_fields_data);
                    } else if ($integration == 'elementor') {
                        $label = $this->get_field_label_from_elementor($field_name, $elementor_fields_data);
                    } else if ($integration == 'forminator') {
                        $label = isset($field['label']) && !empty($field['label']) ? $field['label'] : $field_name;
                    }

                    $modified_fields[] = array(
                        'name' => $field_name,
                        'label' => $label,
                        'type' => $field['type'],
                    );
                }
            }

            if (empty($modified_fields)) {
                return [];
            }

            foreach ($modified_fields as $index => $modified_field) {
                // Remove the field from the modified-fields array if the field is added by 3-rd party plugin
                if (in_array($modified_field['name'], $this->un_supported_fields, true)) {
                    unset($modified_fields[$index]);
                }
            }

            $modified_fields = count($modified_fields) > 0 ? array_values($modified_fields) : [];

            return $modified_fields;
        }

        public function field_name_exists_in_match_columns($field_name, $match_columns, $integration)
        {
            $exists = false;
            if (empty($match_columns)) {
                return $exists;
            }

            foreach ($match_columns as $match_column_info) {
                $field_type = isset($match_column_info['field_type']) ? $match_column_info['field_type'] : '';

                if ($field_type == 'tablesome_smart_fields' || !isset($match_column_info['field_name'])) {
                    continue;
                }

                $configured_field_name = $match_column_info['field_name'];

                if ($integration == 'wpforms') {
                    $field_name = (int) $field_name;
                    $configured_field_name = (int) $match_column_info['field_name'];
                }

                if ($field_name == $configured_field_name) {
                    $exists = true;
                    break;
                }
            }
            return $exists;
        }

        public function get_field_label_from_wpform($field_name, $fields)
        {
            $label = $field_name;
            if (empty($fields)) {
                return $label;
            }
            foreach ($fields as $field) {
                if (isset($field['id']) && $field_name == $field['id']) {
                    $label = $field['label'];
                    break;
                }
            }
            return $label;
        }

        public function get_field_label_from_elementor($field_name, $fields)
        {
            $label = $field_name;
            if (empty($fields)) {
                return $label;
            }
            if (isset($fields[$field_name])) {
                $label = $fields[$field_name]['title'];
            }
            return $label;
        }

        public function the_field_matched_in_manually($field_name, $match_columns)
        {
            $is_matched = false;

            foreach ($match_columns as $match_column_info) {
                $detection_mode = isset($match_column_info['detection_mode']) && !empty($match_column_info['detection_mode']) ? $match_column_info['detection_mode'] : 'auto';
                if (isset($match_column_info['field_name']) && $field_name == $match_column_info['field_name'] && $detection_mode == 'manual') {
                    $is_matched = true;
                    break;
                }
            }
            return $is_matched;
        }

        public function the_field_matched_in_other_triggers($field_name, $args)
        {
            $current_trigger_position = $args['trigger_position'];
            $triggers_meta = $args['triggers_meta'];
            $exists = false;
            $supported_triggers = array(1, 2, 3);

            foreach ($triggers_meta as $trigger_position => $trigger) {

                // Skip, If the trigger position is same as current trigger position
                if ($current_trigger_position == $trigger_position) {
                    continue;
                }

                $trigger_id = isset($trigger['trigger_id']) ? $trigger['trigger_id'] : 0;
                $actions = isset($trigger['actions']) ? $trigger['actions'] : [];

                if (empty($trigger_id) || !in_array($trigger_id, $supported_triggers, true) || empty($actions)) {
                    continue;
                }

                foreach ($actions as $action) {
                    $action_id = $action['action_id'];
                    $match_columns = isset($action['match_columns']) ? $action['match_columns'] : [];

                    if ($action_id != 1 || empty($match_columns)) {
                        continue;
                    }

                    foreach ($match_columns as $match_column_info) {
                        $column_id = isset($match_column_info['column_id']) ? $match_column_info['column_id'] : '';
                        if (isset($match_column_info['field_name']) && $field_name == $match_column_info['field_name'] && !empty($column_id)) {
                            $exists = $column_id;
                            break;
                        }
                    }

                }
            }
            return $exists;
        }

        public function get_column_format_by_field_type($field_type)
        {
            $format = 'text';
            foreach ($this->column_formats as $column_format => $field_types) {
                if (in_array($field_type, $field_types)) {
                    $format = $column_format;
                    break;
                }
            }
            return $format;
        }

        public function modified_smart_fields_columns($args)
        {
            $table = $args['table'];
            $table_id = isset($table->ID) ? $table->ID : 0;
            // return if the table id is 0 (or) missing the trigger & action prop
            if (empty($table) || !isset($args['trigger_position']) || !isset($args['action_position'])) {
                return;
            }
            $trigger_position = intval($args['trigger_position']);
            $action_position = intval($args['action_position']);

            $table_meta = get_tablesome_data($table->ID);
            $triggers_meta = get_tablesome_table_triggers($table->ID);

            $trigger_meta = isset($triggers_meta[$trigger_position]) ? $triggers_meta[$trigger_position] : [];
            $action_meta = isset($trigger_meta['actions'][$action_position]) ? $trigger_meta['actions'][$action_position] : array();
            if (empty($action_meta)) {
                return;
            }
            $match_columns = isset($action_meta['match_columns']) ? $action_meta['match_columns'] : array();
            if (empty($match_columns)) {
                return;
            }

            $last_column_id = $table_meta['meta']['last_column_id'];
            $new_column_ids = [];
            foreach ($match_columns as $match_column_index => $match_column) {
                $field_type = isset($match_column['field_type']) ? $match_column['field_type'] : '';
                $detection_mode = isset($match_column['detection_mode']) ? $match_column['detection_mode'] : 'auto';
                $column_status = isset($match_column['column_status']) ? $match_column['column_status'] : '';
                $column_id = isset($match_column['column_id']) ? $match_column['column_id'] : '';

                if ($field_type != 'tablesome_smart_fields') {
                    continue;
                }

                if ($column_status == 'pending' && $column_id == 0 && $detection_mode == 'enabled') {
                    $last_column_id = $last_column_id + 1;
                    $smart_field = get_tablesome_smart_field_info_by_field_name($match_column['field_name']);

                    $table_meta['columns'][] = array(
                        'id' => $last_column_id,
                        'name' => $smart_field['column_label'],
                        'format' => $smart_field['column_format'],
                    );

                    // Collect the newely created column-ids and their corresponding property index
                    $new_column_ids[] = array(
                        'column_id' => $last_column_id,
                        'match_column_index' => $match_column_index,
                    );
                }
            }

            if (empty($new_column_ids)) {
                return;
            }

            $table_meta['meta']['last_column_id'] = $last_column_id;

            $table_meta = set_tablesome_data($table_id, $table_meta);

            // Store new fields in DB.
            $helper = new \Tablesome\Includes\Lib\Table_Crud_WP\Helper();
            $table_columns = $helper->get_table_columns($table_meta);

            $db_table = new \Tablesome_Table(array(
                'table_name' => "tablesome_table_{$table_id}",
            ));
            $db_table->modify_the_table($table_meta, $table_columns, []);

            foreach ($new_column_ids as $new_column_data) {
                $column_name = "column_{$new_column_data['column_id']}";
                $column_id_exists = $db_table->column_exists($column_name);
                if ($column_id_exists) {
                    $trigger_meta['actions'][$action_position]['match_columns'][$new_column_data['match_column_index']]['column_id'] = $new_column_data['column_id'];
                    $trigger_meta['actions'][$action_position]['match_columns'][$new_column_data['match_column_index']]['column_status'] = 'published';
                }
            }

            $triggers_meta[$trigger_position] = []; // reset the trigger meta by trigger position
            $triggers_meta[$trigger_position] = $trigger_meta; // update the trigger meta by trigger position
            $triggers_meta = set_tablesome_table_triggers($table_id, $triggers_meta);

        }

        public function get_smart_fields_values()
        {
            $current_datetime = date('Y-m-d H:i:s');
            $unix_timestamp = strtotime($current_datetime);

            $values = array(
                'ip_address' => get_tablesome_ip_address(),
                'page_source_url' => get_tablesome_request_url(),
                'created_at_datetime' => $current_datetime,
                'created_at' => $unix_timestamp * 1000,
                'created_by' => get_current_user_id(),
            );

            return $values;
        }
    }
}