<?php

namespace Tablesome\Includes\Modules\Automation;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Tracking')) {
    class Tracking
    {

        public function get_events()
        {
            $events = [];
            $status = $this->get_triggers_and_actions_status();
            $events['triggers_and_actions_used'] = $status;

            if (empty($status)) {
                return $events;
            }

            $integration_counts = $this->get_trigger_integration_counts();

            $events['triggers_collection'] = $integration_counts['triggers_collection'];
            $events['actions_collection'] = $integration_counts['actions_collection'];
            return $events;
        }

        /**
         * Use of the below method for checking users are using the triggers and actions feature or not.
         * If return true then the user is using the triggers and actions feature.
         * If return false then the user is not using the triggers and actions feature.
         */
        public function get_triggers_and_actions_status()
        {
            global $wpdb;

            $query = "SELECT count(pm.post_id) as records_count FROM {$wpdb->postmeta} pm LEFT JOIN {$wpdb->posts} p ";
            $query .= "ON p.ID = pm.post_id WHERE p.post_type = 'tablesome_cpt' AND p.post_status='publish' AND pm.meta_key = 'tablesome_table_triggers'";

            $result = $wpdb->get_var($query);

            return $result > 0 ? true : false;
        }

        public function get_trigger_integration_counts()
        {
            $tables = Triggers::get_instance()->get_tables();
            if (!isset($tables) || empty($tables)) {
                return [];
            }

            $triggers_collection = array();
            $actions_collection = array();

            foreach ($tables as $table) {

                $triggers_meta = get_tablesome_table_triggers($table->ID);

                if (!isset($triggers_meta) || empty($triggers_meta)) {
                    continue;
                }

                foreach ($triggers_meta as $trigger_meta) {

                    $integration_name_of_trigger = isset($trigger_meta['integration']) ? $trigger_meta['integration'] : '';

                    $actions = isset($trigger_meta['actions']) ? $trigger_meta['actions'] : [];

                    foreach ($actions as $action_meta) {
                        $integration_name_of_action = isset($action_meta['integration']) ? $action_meta['integration'] : 'tablesome';
                        $action_id = isset($action_meta['action_id']) ? $action_meta['action_id'] : 0;
                        $action_name = $this->get_action_name_by_id($action_id);

                        // Collect what are the triggers they are using it and their count.
                        if (!isset($triggers_collection[$integration_name_of_trigger])) {
                            $triggers_collection[$integration_name_of_trigger]['count'] = 1;
                            $triggers_collection[$integration_name_of_trigger]['integrated'] = 1;
                        }

                        if (isset($triggers_collection[$integration_name_of_trigger])) {
                            $triggers_collection[$integration_name_of_trigger]['count']++;
                            $triggers_collection[$integration_name_of_trigger]['integrated'] = 1;
                        }

                        // Collect what are the integrations they are using and their count.
                        if (!isset($actions_collection[$integration_name_of_action])) {
                            $actions_collection[$integration_name_of_action]['count'] = 1;
                            $actions_collection[$integration_name_of_action]['active'] = 1;
                        }

                        if (isset($actions_collection[$integration_name_of_action])) {
                            $actions_collection[$integration_name_of_action]['count']++;
                            $actions_collection[$integration_name_of_action]['active'] = 1;
                        }

                        // Collect actions count. how many actions they are using in entire site.
                        if (!isset($actions_collection[$integration_name_of_action][$action_name])) {
                            $actions_collection[$integration_name_of_action][$action_name] = 1;
                        }

                        if (isset($actions_collection[$integration_name_of_action][$action_name])) {
                            $actions_collection[$integration_name_of_action][$action_name]++;
                        }

                    }
                }
            }

            return array(
                'triggers_collection' => $triggers_collection,
                'actions_collection' => $actions_collection,
            );
        }

        public function get_action_name_by_id($action_id)
        {
            $name = 'unknown_action';
            if (empty($action_id)) {
                return $name;
            }
            $actions = apply_filters('tablesome_automation/the_actions', array());
            foreach ($actions as $data) {
                foreach ($data['actions'] as $action_meta) {
                    if ($action_id == $action_meta['id']) {
                        $name = $action_meta['name'];
                        break;
                    }
                }
            }

            return $name;
        }
    }
}
