<?php

namespace Tablesome\Includes\Modules\Automation\Traits;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

use \Tablesome\Includes\Settings\Tablesome_Getter;

trait Store_All_Forms_Entries
{

    public function __construct()
    {
        add_action('tablesome_automation/store_all_forms_entries', [$this, 'init'], 10, 2);
    }

    public function init($args = array())
    {
        $enabled_all_forms_entries = Tablesome_Getter::get('enabled_all_forms_entries');
        $form_id = isset($args['form_id']) ? $args['form_id'] : '';
        if (!$enabled_all_forms_entries || empty($form_id)) {
            return;
        }

        global $wpdb;
        $post_metas = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT p.ID,pm.meta_value
                        FROM $wpdb->postmeta pm
                            LEFT JOIN $wpdb->posts p
                                ON p.ID = pm.post_id
                        WHERE p.post_type IS NOT NULL
                        AND p.post_status = %s
                        AND pm.meta_key = %s
                        AND pm.`meta_value` LIKE %s",
                'publish',
                'tablesome_table_triggers',
                '%%tablesome%%'
            )
        );
        // If the records are empty, should create a new table with the default form trigger data
        if (empty($post_metas)) {
            $this->create_table_with_default_form_triggers($args);
        }

        if (!empty($post_metas)) {

            $form_already_configured = true;
            foreach ($post_metas as $post_meta) {
                $table_id = $post_meta->ID;
                $triggers_meta = isset($post_meta->meta_value) && !empty($post_meta->meta_value) ? maybe_unserialize($post_meta->meta_value) : [];

                $form_ids = isset($triggers_meta) && !empty($triggers_meta) ? array_column($triggers_meta, 'integration', 'form_id') : [];

                if (!isset($form_ids[$form_id])) {
                    $form_already_configured = false;
                }

                if (isset($form_ids[$form_id]) && !$form_already_configured) {
                    $form_already_configured = true;
                    break;
                }

            }

            if (!$form_already_configured) {
                $this->create_table_with_default_form_triggers($args);
            }
        }

    }

    public function create_table($args = array())
    {
        $table = new \Tablesome\Includes\Core\Table();

        $post_data = array(
            'post_title' => $args['form_title'],
            'post_type' => TABLESOME_CPT,
            'post_content' => '',
            'post_status' => 'publish',
        );

        return $table->insert_or_update_post(0, $post_data);
    }

    public function get_default_form_triggers($table_id, $args)
    {
        $default_smart_fields_data = get_default_tablesome_smart_fields();
        $form_id = $args['integration'] != 'elementor' ? (int) $args['form_id'] : $args['form_id'];

        $triggers = array(
            array(
                'integration' => $args['integration'],
                'status' => 1,
                'trigger_id' => $this->get_form_trigger_id_by_integration($args['integration']),
                'form_id' => $form_id,
                'actions' => array(
                    array(
                        'integration' => 'tablesome',
                        'match_columns' => $default_smart_fields_data,
                        'autodetect_enabled' => 1,
                        'action_id' => 1,
                    ),
                ),
            ),
        );
        return $triggers;
    }

    public function create_table_with_default_form_triggers($args)
    {
        $table_id = $this->create_table($args);
        $table_meta = set_tablesome_data($table_id,
            array(
                'columns' => array(),
                'meta' => array(
                    'last_column_id' => 0,
                ),
            )
        );
        $form_triggers = $this->get_default_form_triggers($table_id, $args);
        set_tablesome_table_triggers($table_id, $form_triggers);

        $tablesome_db = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();
        $tablesome_db->create_table_instance($table_id, $table_meta);
    }

    public function get_form_trigger_id_by_integration($integration)
    {
        $triggers = apply_filters('tablesome_automation/the_triggers', array());
        $trigger_id = 0;
        foreach ($triggers as $trigger) {
            if ($trigger['integration'] == $integration && $trigger['trigger'] == "tablesome_{$integration}_form_submit") {
                $trigger_id = $trigger['trigger_id'];
                break;
            }
        }
        return $trigger_id;
    }
}
