<?php

namespace Tablesome\Includes\Modules\Automation;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

use Tablesome\Includes\Modules\Automation\Integrations\WP_Forms;
use \Tablesome\Includes\Modules\Automation\Actions;
use \Tablesome\Includes\Modules\Automation\Integrations\Cf7;
use \Tablesome\Includes\Modules\Automation\Integrations\Elementor;
use \Tablesome\Includes\Modules\Automation\Integrations\Forminator;
use \Tablesome\Includes\Modules\Automation\Traits\Store_All_Forms_Entries;

if (!class_exists('\Tablesome\Includes\Modules\Automation\Triggers')) {
    class Triggers
    {
        use Store_All_Forms_Entries;

        public static $instance = null;
        public static $action_instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_triggers();
                self::$instance->add_trigger_hook();
                self::$action_instance = Actions::get_instance();
            }
            return self::$instance;
        }

        public function load_triggers()
        {
            Cf7::get_instance();
            WP_Forms::get_instance();
            Elementor::get_instance();
            Forminator::get_instance();
        }

        public function add_trigger_hook()
        {
            add_action('tablesome_automation/trigger', [self::$instance, 'run_triggers'], 10, 2);
        }

        public function run_triggers(array $form_submitted_data = array())
        {
            $form_id = isset($form_submitted_data['form_submission_id']) ? $form_submitted_data['form_submission_id'] : 0;
            $posted_data = isset($form_submitted_data['form_submission_data']) ? $form_submitted_data['form_submission_data'] : [];
            $integration = isset($form_submitted_data['integration']) ? $form_submitted_data['integration'] : '';
            $form_title = isset($form_submitted_data['form_title']) ? $form_submitted_data['form_title'] : 'Untitled Form';

            if (empty($form_id) || empty($posted_data)) {
                return;
            }

            do_action('tablesome_automation/store_all_forms_entries', array(
                'form_id' => $form_id,
                'form_title' => $form_title,
                'integration' => $integration,
            ));

            $tables = $this->get_tables();
            if (!isset($tables) || empty($tables)) {
                return;
            }

            $is_premium = tablesome_fs()->can_use_premium_code__premium_only();

            foreach ($tables as $table) {

                $triggers_meta = get_tablesome_table_triggers($table->ID);
                $table_meta = get_tablesome_data($table->ID);
                if (!isset($triggers_meta) || empty($triggers_meta)) {
                    continue;
                }

                $user_can_access_tablesome_trigger = user_can_access_tablesome_trigger($triggers_meta);
                if (!$user_can_access_tablesome_trigger) {
                    continue;
                }

                $trigger_position = get_tablesome_trigger_position($triggers_meta);
                // error_log('$trigger_position : ' . $trigger_position);

                foreach ($triggers_meta as $trigger_index => $trigger_meta) {

                    $can_start_trigger = ($is_premium || (!$is_premium && $trigger_index == $trigger_position)) ? true : false;
                    // error_log('$can_start_trigger : ' . $can_start_trigger);

                    $is_valid_trigger = $this->is_valid_trigger($trigger_meta);

                    $trigger_form_id = isset($trigger_meta['form_id']) ? $trigger_meta['form_id'] : 0;

                    $submitted_form_configured = ($form_id == $trigger_form_id);

                    if (!$is_valid_trigger || !$submitted_form_configured || !$can_start_trigger) {
                        continue;
                    }

                    $actions = isset($trigger_meta['actions']) ? $trigger_meta['actions'] : [];

                    if (empty($actions)) {continue;}

                    $user_can_access_tablesome_action = user_can_access_tablesome_action($actions);
                    // error_log('$user_can_access_tablesome_action : ' . $user_can_access_tablesome_action);
                    if (!$user_can_access_tablesome_action) {
                        continue;
                    }

                    $free_actions_positions = get_tablesome_free_actions_positions($actions);
                    // error_log('$action_position : ' . $action_position);

                    foreach ($actions as $action_index => $action_meta) {

                        $can_start_action = ($is_premium || !$is_premium && in_array($action_index, $free_actions_positions)) ? true : false;
                        // error_log('$can_start_action : ' . $can_start_action);

                        if (!$can_start_action) {
                            continue;
                        }

                        self::$action_instance->run_actions([
                            "trigger_position" => $trigger_index,
                            "action_position" => $action_index,
                            "table" => $table,
                            "table_meta" => $table_meta,
                            "action_meta" => $action_meta,
                            "trigger_meta" => $trigger_meta,
                            "form_submitted_data" => $form_submitted_data,
                        ]);
                    }
                }
            }
        }

        /**
         * trigger must have a action. If not, it is not consider as valid trigger.
         */
        public function is_valid_trigger($trigger_meta)
        {
            $trigger_actions_meta = isset($trigger_meta['actions']) ? $trigger_meta['actions'] : [];
            if (empty($trigger_actions_meta)) {
                return false;
            }

            $trigger_is_active = isset($trigger_meta['status']) && $trigger_meta['status'] == 1 ? true : false;

            if (!$trigger_is_active) {
                return false;
            }

            if (!$this->is_valid_trigger_by_id($trigger_meta['trigger_id'])) {
                return false;
            }

            return true;
        }

        public function get_tables()
        {
            $tables = get_posts(
                array(
                    'post_type' => TABLESOME_CPT,
                    'numberposts' => -1,
                )
            );

            return $tables;
        }

        private function is_valid_trigger_by_id($trigger_id)
        {
            $is_valid = false;
            $triggers = apply_filters('tablesome_automation/the_triggers', array());
            foreach ($triggers as $trigger) {
                $is_valid = ($trigger['trigger_id'] == $trigger_id);
                if ($is_valid) {
                    break;
                }
            }
            return $is_valid;
        }

        public function get_trigger_name_by_id($trigger_id)
        {
            $triggers = apply_filters('tablesome_automation/the_triggers', array());
            foreach ($triggers as $trigger) {
                if ($trigger['trigger_id'] == $trigger_id) {
                    return $trigger['trigger_label'];
                }
            }
            return '';
        }

        public function get_trigger_integration_name_by_id($trigger_id)
        {
            $triggers = apply_filters('tablesome_automation/the_triggers', array());
            foreach ($triggers as $trigger) {
                if ($trigger['trigger_id'] == $trigger_id) {
                    return $trigger['integration_label'];
                }
            }
            return '';
        }
    }
}
