<?php
namespace Tablesome\Includes\Settings;

if (!class_exists('\Tablesome\Includes\Settings\Typography')) {
    class Typography
    {
        public static $webfonts = array();

        public static function get_google_fonts()
        {
            include_once TABLESOME_PATH . 'includes/settings/typography/google-fonts.php';
            return tablesome_get_google_fonts();
        }

        public static function add_typography_enqueue_styles()
        {

            if (!empty(self::$webfonts)) {

                if (!empty(self::$webfonts['enqueue'])) {

                    $query = array();
                    $fonts = array();

                    foreach (self::$webfonts['enqueue'] as $family => $styles) {
                        $fonts[] = $family . ((!empty($styles)) ? ':' . implode(',', $styles) : '');
                    }

                    if (!empty($fonts)) {
                        $query['family'] = implode('%7C', $fonts);
                    }

                    if (!empty(self::$subsets)) {
                        $query['subset'] = implode(',', self::$subsets);
                    }

                    $query['display'] = 'swap';

                    // error_log(' query : ' . print_r($query, true));

                    wp_enqueue_style('csf-google-web-fonts', esc_url(add_query_arg($query, '//fonts.googleapis.com/css')), array(), null);

                }

                if (!empty(self::$webfonts['async'])) {

                    $fonts = array();

                    foreach (self::$webfonts['async'] as $family => $styles) {
                        $fonts[] = $family . ((!empty($styles)) ? ':' . implode(',', $styles) : '');
                    }

                    wp_enqueue_script('csf-google-web-fonts', esc_url('//ajax.googleapis.com/ajax/libs/webfont/1.6.26/webfont.js'), array(), null);

                    wp_localize_script('csf-google-web-fonts', 'WebFontConfig', array('google' => array('families' => $fonts)));

                }

            }

        }

        public function enqueue($bundle_name)
        {

            include_once TABLESOME_PATH . 'includes/settings/typography/google-fonts.php';
            // return tablesome_get_google_fonts();

            // if (!wp_script_is('csf-webfontloader')) {

            // CSF::include_plugin_file('fields/typography/google-fonts.php');

            wp_enqueue_script('csf-webfontloader', 'https://cdn.jsdelivr.net/npm/webfontloader@1.6.28/webfontloader.min.js', [], '1.6.28', true);

            $webfonts = array();

            $customwebfonts = apply_filters('csf_field_typography_customwebfonts', array());

            if (!empty($customwebfonts)) {
                $webfonts['custom'] = array(
                    'label' => esc_html__('Custom Web Fonts', 'csf'),
                    'fonts' => $customwebfonts,
                );
            }

            $webfonts['safe'] = array(
                'label' => esc_html__('Safe Web Fonts', 'csf'),
                'fonts' => apply_filters('csf_field_typography_safewebfonts', array(
                    'Arial',
                    'Arial Black',
                    'Helvetica',
                    'Times New Roman',
                    'Courier New',
                    'Tahoma',
                    'Verdana',
                    'Impact',
                    'Trebuchet MS',
                    'Comic Sans MS',
                    'Lucida Console',
                    'Lucida Sans Unicode',
                    'Georgia, serif',
                    'Palatino Linotype',
                )
                ));

            $webfonts['google'] = array(
                'label' => esc_html__('Google Web Fonts', 'csf'),
                'fonts' => apply_filters('csf_field_typography_googlewebfonts', tablesome_get_google_fonts()
                ));

            $defaultstyles = apply_filters('csf_field_typography_defaultstyles', array('normal', 'italic', '700', '700italic'));

            $googlestyles = apply_filters('csf_field_typography_googlestyles', array(
                '100' => 'Thin 100',
                '100italic' => 'Thin 100 Italic',
                '200' => 'Extra-Light 200',
                '200italic' => 'Extra-Light 200 Italic',
                '300' => 'Light 300',
                '300italic' => 'Light 300 Italic',
                'normal' => 'Normal 400',
                'italic' => 'Normal 400 Italic',
                '500' => 'Medium 500',
                '500italic' => 'Medium 500 Italic',
                '600' => 'Semi-Bold 600',
                '600italic' => 'Semi-Bold 600 Italic',
                '700' => 'Bold 700',
                '700italic' => 'Bold 700 Italic',
                '800' => 'Extra-Bold 800',
                '800italic' => 'Extra-Bold 800 Italic',
                '900' => 'Black 900',
                '900italic' => 'Black 900 Italic',
            ));

            $webfonts = apply_filters('csf_field_typography_webfonts', $webfonts);

            wp_localize_script($bundle_name, 'csf_typography_json', array(
                'webfonts' => $webfonts,
                'defaultstyles' => $defaultstyles,
                'googlestyles' => $googlestyles,
            ));

            // }

        }
    }
}
