<?php

namespace Tablesome\Includes\Tracking;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('\Tablesome\Includes\Tracking\Model')) {
    class Model
    {
        public function get_data()
        {
            $crud = new \Tablesome\Includes\Db\CRUD();

            $automation_tracking_controller = new \Tablesome\Includes\Modules\Automation\Tracking();

            $options_data = $this->get_options_data();
            $data = $this->get_required_fields_data_from_setting($options_data);
            $args = $this->get_exclude_tables_args();

            $table_collection_data = $crud->get_tables_collection($args);

            $triggers_events_data = $automation_tracking_controller->get_events();

            // $themes_and_plguins_data = $this->get_themes_and_plugins_data();

            $data = array_merge($data, $table_collection_data, $triggers_events_data);

            /**
             * Compare new data with the old data and return the difference
             */
            $data = $this->get_differanciated_data($data);

            $data = $this->remove_not_needed_data($data);

            return $data;
        }

        public function remove_not_needed_data($data)
        {
            foreach (array('num_of_records_per_page', 'hide_table_header', 'sorting', 'min_column_width', 'plugins_info', 'themes_info') as $event_name) {
                if (isset($data[$event_name])) {
                    unset($data[$event_name]);
                }
            }
            return $data;
        }

        public function get_exclude_tables_args()
        {
            $option_name = TABLESOME_SAMPLE_TABLE_OPTION;
            $sample_table_id = \get_option($option_name);

            if (empty($sample_table_id)) {
                return [];
            }
            return array(
                'post__not_in' => array($sample_table_id),
            );
        }

        public function get_options_data()
        {
            $option_name = TABLESOME_OPTIONS;
            return \get_option($option_name);
        }

        public function get_required_fields()
        {
            return array(
                // "num_of_records_per_page",
                // "show_serial_number_column",
                "search",
                // "hide_table_header",
                // "sorting",
                "filters",
                "mobile_layout_mode",
                "style_disable",
                // "min_column_width",
            );
        }

        public function get_required_fields_data_from_setting($data)
        {
            $required_fields = $this->get_required_fields();

            foreach ($data as $key => $value) {
                if (!in_array($key, $required_fields)) {
                    unset($data[$key]);
                }
            }
            return $data;
        }

        public function get_user_props()
        {
            $fs_utils = new \Tablesome\Includes\Freemius_Utils();
            $fs_collection_props = $fs_utils->get_collection_props();
            $props = array();
            $props['plan'] = $fs_collection_props['plan'];
            $props['wp_version'] = $fs_collection_props['wp_version'];
            $props['php_version'] = $fs_collection_props['php_version'];
            $props['site_url'] = $fs_collection_props['site_url'];
            $props['email'] = $fs_collection_props['email'];
            $current_user = wp_get_current_user();
            if (isset($current_user) && !empty($current_user)) {
                $data = isset($current_user->data) ? $current_user->data : null;
                $props['login_name'] = isset($data->user_login) ? $data->user_login : '';
                $props['role'] = isset($current_user->roles[0]) ? $current_user->roles[0] : '';
            }
            return $props;
        }

        public function get_browser_name($user_agent)
        {
            if (strpos($user_agent, 'Opera') || strpos($user_agent, 'OPR/')) {
                return 'Opera';
            } elseif (strpos($user_agent, 'Edge')) {
                return 'Edge';
            } elseif (strpos($user_agent, 'Chrome')) {
                return 'Chrome';
            } elseif (strpos($user_agent, 'Safari')) {
                return 'Safari';
            } elseif (strpos($user_agent, 'Firefox')) {
                return 'Firefox';
            } elseif (strpos($user_agent, 'MSIE') || strpos($user_agent, 'Trident/7')) {
                return 'Internet Explorer';
            }

            return 'Other';
        }

        public function get_differanciated_data($current_events_data)
        {
            $last_time_stored_insights_data = get_tablesome_insights_data();

            $differanciated_data = tablesome_multi_array_diff_assoc($current_events_data, $last_time_stored_insights_data);

            $storing_data = array_merge($current_events_data, $differanciated_data);

            /**
             * Store current data
             */
            set_tablesome_insights_data($storing_data);

            return $differanciated_data;
        }

        public function get_themes_and_plugins_data()
        {
            return array(
                'plugins_info' => $this->get_plugins_info(),
                'themes_info' => $this->get_themes_info(),
            );
        }

        private function get_plugins_info()
        {
            $plugins = array();
            $helpers = new \Tablesome\Includes\Helpers();
            $all_plugins = $helpers->get_plugins_data();

            if (empty($all_plugins)) {
                return $plugins;
            }

            foreach ($all_plugins as $plugin) {
                $domain = $plugin['TextDomain'];

                $plugins[$domain] = array(
                    'name' => $plugin['Name'],
                    'version' => $plugin['Version'],
                    'is_active' => $plugin['is_active'],
                    'author' => $plugin['Author'],
                );
            }

            return $plugins;
        }

        private function get_themes_info()
        {
            $themes = array();
            $all_themes = wp_get_themes();

            if (empty($all_themes)) {
                return $themes;
            }

            $current_theme = wp_get_theme();
            $current_theme_domain = $current_theme->get('TextDomain');

            foreach ($all_themes as $theme) {
                $domain = $theme->get('TextDomain');
                $is_active = ($domain == $current_theme_domain);

                $themes[$domain] = array(
                    'name' => $theme->get('Name'),
                    'version' => $theme->get('Version'),
                    'is_active' => $is_active ? 1 : 0,
                );
            }
            return $themes;
        }
    }
}